@@ -93,6 +93,7 @@ BEGIN {
           pipe_tee
           untaint
           $PKGNAME_REGEX
+          $PKGREPACK_REGEX
           $PKGVERSION_REGEX
           ),
         @{ $EXPORT_TAGS{constants} });
@@ -216,6 +217,15 @@ is not anchored and does not enforce any "boundary" characters.
 
 our $PKGNAME_REGEX = qr/[a-z0-9][-+\.a-z0-9]+/o;
 
+=item $PKGREPACK_REGEX
+
+Regular expression that matches "repacked" package names.  The expression is
+not anchored and does not enforce any "boundary" characters.
+
+=cut
+
+our $PKGREPACK_REGEX = qr/(dfsg|debian|ds|repack)/o;
+
 =item $PKGVERSION_REGEX
 
 Regular expression that matches valid package versions.  The
@@ -427,7 +437,7 @@ sub visit_dpkg_paragraph {
         # empty line?
         if ($_ eq '' || (!$debconf && m/^\s*$/)) {
             if ($open_section) { # end of current section
-                # pass the current section to the handler
+                 # pass the current section to the handler
                 $code->($section, $lines);
                 $section = {};
                 $lines = {};
@@ -894,8 +904,16 @@ sub do_fork() {
             }
         }
     }
-    sigprocmask(SIG_SETMASK, $orig_mask, undef)
-      or die("sigprocmask failed: $!\n");
+    if (!sigprocmask(SIG_SETMASK, $orig_mask, undef)) {
+        # The child MUST NOT use die as the caller cannot distinguish
+        # the caller from the child via an exception.
+        my $sigproc_error = $!;
+        if (not defined($pid) or $pid != 0) {
+            die("sigprocmask failed (do_fork, parent): $sigproc_error\n");
+        }
+        print STDERR "sigprocmask failed (do_fork, child): $sigproc_error\n";
+        POSIX::_exit(255);
+    }
     if (not defined($pid)) {
         $! = $fork_error;
     }
@@ -987,7 +1005,7 @@ sub perm2oct {
                     ([-r])([-w])([-xsS])     # group
                     ([-r])([-w])([-xtT])     # other
                /xsmo
-      ) {
+    ) {
         croak "$t does not appear to be a permission string";
     }
 
@@ -1720,11 +1738,23 @@ gcc-5 + gcc-5-cross + gcc-6 + gcc-6-cross).
 
 sub find_backlog {
     my ($lintian_version, $state) = @_;
-    my (@backlog, %by_version);
+    my (@backlog, %by_version, @low_priority);
     for my $group_id (keys(%{$state->{'groups'}})) {
         my $last_version = '0';
         my $group_data = $state->{'groups'}{$group_id};
         my $is_out_of_date;
+        # Does this group repeatedly fail with the current version
+        # of lintian?
+        if (    exists($group_data->{'processing-errors'})
+            and $group_data->{'processing-errors'} > 2
+            and exists($group_data->{'last-error-by'})
+            and $group_data->{'last-error-by'} ne $lintian_version) {
+            # To avoid possible "starvation", we will give lower priority
+            # to packages that repeatedly fail.  They will be retried as
+            # the backlog is cleared.
+            push(@low_priority, $group_id);
+            next;
+        }
         if (exists($group_data->{'out-of-date'})) {
             $is_out_of_date = $group_data->{'out-of-date'};
         }
@@ -1738,6 +1768,7 @@ sub find_backlog {
     for my $v (sort(versions_comparator keys(%by_version))) {
         push(@backlog, shuffle(@{$by_version{$v}}));
     }
+    push(@backlog, shuffle(@low_priority)) if @low_priority;
     return @backlog;
 }
 
